'use strict';

exports.__esModule = true;

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _getFieldHandler = require('./modules/getFieldHandler');

var _getNextData = require('./modules/getNextData');

var _buildErrorMessage = require('./modules/buildErrorMessage');

var _buildErrorMessage2 = _interopRequireDefault(_buildErrorMessage);

var _LayerInterface = require('./LayerInterface');

var _LayerInterface2 = _interopRequireDefault(_LayerInterface);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

/**
 * @desc Stateless container of DOM event handlers:
 *         - onChange
 *         - onBlur
 *         - onSubmit
 *
 */
var LayerDomHandlers = function (_React$Component) {
  _inherits(LayerDomHandlers, _React$Component);

  function LayerDomHandlers() {
    var _temp, _this, _ret;

    _classCallCheck(this, LayerDomHandlers);

    for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }

    return _ret = (_temp = (_this = _possibleConstructorReturn(this, _React$Component.call.apply(_React$Component, [this].concat(args))), _this), _this.handleDomChange = function (event) {
      var _this2 = _this,
          props = _this2.props;

      var domData = (0, _getNextData.getNextDataFromDom)(props.layerId, props.data, event);

      _this.handleChange(domData);
    }, _this.handleCustomChange = function (fieldId, value) {
      var _this3 = _this,
          props = _this3.props;

      var domData = (0, _getNextData.getNextDataFromCustomHandler)(fieldId, value, props.data);

      _this.handleChange(domData);
    }, _this.handleChange = function (domData) {
      var _this4 = _this,
          props = _this4.props;
      var stateContainer = props.stateContainer;


      var field = stateContainer.getNormalizedField(domData.fieldId);
      var filter = (0, _getFieldHandler.getFieldValueHandler)(props.layerId, field.id, 'filter', field.filter, props.filter);

      if (filter // if filter is provided
      && domData.value // & there's a value is not an empty string
      && !filter(domData.value, props.data) // & filter function returned false
      ) {
          return; // then ignoring this update
        }

      var transformBeforeStore = (0, _getFieldHandler.getFieldValueHandler)(props.layerId, field.id, 'transformBeforeStore', field.transformBeforeStore, props.transformBeforeStore);

      var processedValue = transformBeforeStore ? transformBeforeStore(domData.value, props.data) : domData.value;

      var processedDomData = _extends({}, domData, { value: processedValue });

      if (domData.event) {
        var handleChange = (0, _getFieldHandler.getFieldDomHandler)(props.layerId, field.id, 'onChange', field.handlers, props.handlers);

        // $FlowIgnoreMe: Even getFieldDomHandler throws if onChange is missing, flow can't infer it
        handleChange(processedDomData);
      }

      stateContainer.validateField(field, processedDomData);
    }, _this.handleDomBlur = function (event) {
      var _this5 = _this,
          props = _this5.props;

      var domData = (0, _getNextData.getNextDataFromDom)(props.layerId, props.data, event);

      _this.handleBlur(domData);
    }, _this.handleCustomBlur = function (fieldId, value, event) {
      var _this6 = _this,
          props = _this6.props;

      var domData = (0, _getNextData.getNextDataFromCustomHandler)(fieldId, value, props.data, event);

      _this.handleBlur(domData, true);
    }, _this.handleBlur = function (domData) {
      var isCustom = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : false;
      var _this7 = _this,
          props = _this7.props;
      var stateContainer = props.stateContainer;


      var field = stateContainer.getNormalizedField(domData.fieldId);
      var handleBlur = (0, _getFieldHandler.getFieldDomHandler)(props.layerId, field.id, 'onBlur', field.handlers, props.handlers);

      var transformBeforeStore = (0, _getFieldHandler.getFieldValueHandler)(props.layerId, field.id, 'transformBeforeStore', field.transformBeforeStore, props.transformBeforeStore);

      var processedValue = transformBeforeStore ? transformBeforeStore(domData.value, props.data) : domData.value;

      var processedDomData = _extends({}, domData, { value: processedValue });

      if (!isCustom && handleBlur) handleBlur(processedDomData);

      stateContainer.validateField(field, processedDomData);
    }, _this.handleSubmit = function (event) {
      if (event && event.preventDefault) {
        event.preventDefault();
      }

      _this.props.stateContainer.triggerSubmission();
    }, _temp), _possibleConstructorReturn(_this, _ret);
  }

  /**
   * @desc Change handlers.
   *
   */


  /**
   * @desc Blur handlers.
   *
   */


  /**
   * @desc Submit handler.
   *
   */


  LayerDomHandlers.prototype.render = function render() {
    var props = this.props;


    var layer = new _LayerInterface2.default(props.stateContainer, {
      handleDomBlur: this.handleDomBlur,
      handleDomChange: this.handleDomChange,
      handleCustomBlur: this.handleCustomBlur,
      handleCustomChange: this.handleCustomChange,
      handleSubmit: this.handleSubmit
    });

    if (typeof props.children !== 'function') {
      throw new Error((0, _buildErrorMessage2.default)({
        layerId: props.layerId,
        message: '`children` must be a function'
      }));
    }

    return props.children(layer);
  };

  return LayerDomHandlers;
}(_react2.default.Component);

exports.default = LayerDomHandlers;