'use strict';

exports.__esModule = true;

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _Constant = require('./enums/Constant');

var _Constant2 = _interopRequireDefault(_Constant);

var _ids = require('./modules/ids');

var _buildErrorMessage = require('./modules/buildErrorMessage');

var _buildErrorMessage2 = _interopRequireDefault(_buildErrorMessage);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

/**
 * @desc Interface to access validation layer data from the views.
 *
 */
var LayerInterface = function LayerInterface(stateContainer, handlers) {
  var _this = this;

  _classCallCheck(this, LayerInterface);

  this.getPropsFor = function (keyPath) {
    var field = _this.__getField(keyPath, 'getPropsFor');

    // $FlowIssue: exact type + destructuring: https://github.com/facebook/flow/issues/2405
    return _extends({}, field.props, {
      onBlur: _this.__handleDomBlur,
      onChange: _this.__handleDomChange
    });
  };

  this.getCheckboxPropsFor = function (keyPath) {
    var field = _this.__getField(keyPath, 'getCheckboxPropsFor');

    // $FlowIssue: exact type + destructuring: https://github.com/facebook/flow/issues/2405
    return _extends({}, field.props, {
      checked: !!field.props.value,
      onBlur: _this.__handleDomBlur,
      onChange: _this.__handleDomChange
    });
  };

  this.getRadioButtonPropsFor = function (keyPath, value) {
    var layerId = _this.__layerId;
    var field = _this.__getField(keyPath, 'getRadioButtonPropsFor');

    // $FlowIssue: exact type + destructuring: https://github.com/facebook/flow/issues/2405
    return _extends({}, field.props, {
      value: value,
      id: (0, _ids.buildFieldDomIdWithValue)(layerId, keyPath, value),
      checked: field.props.value === value,
      onBlur: _this.__handleDomBlur,
      onChange: _this.__handleDomChange
    });
  };

  this.getCustomPropsFor = function (keyPath, _ref) {
    var value = _ref.value,
        disabled = _ref.disabled,
        getChecked = _ref.getChecked;

    var layerId = _this.__layerId;
    var field = _this.__getField(keyPath, 'getCustomPropsFor');

    var customProps = _extends({}, field.props, {
      onBlur: _this.__handleDomBlur,
      onChange: _this.__handleDomChange
    });

    if (value) {
      customProps.id = (0, _ids.buildFieldDomIdWithValue)(layerId, keyPath, value);
    }

    if (disabled) {
      customProps.disabled = disabled;
    }

    if (getChecked) {
      customProps.checked = getChecked(field.props.value);
    }

    // $FlowIssue: exact type + destructuring: https://github.com/facebook/flow/issues/2405
    return customProps;
  };

  this.getSubmitButtonProps = function () {
    return {
      type: 'submit',
      disabled: _this.__isSubmitting
    };
  };

  this.getValidityFor = function (keyPath) {
    var field = _this.__getField(keyPath, 'getValidityFor');
    return field.resolution && typeof field.resolution.valid !== 'undefined' ? field.resolution.valid : null;
  };

  this.getStatusFor = function (keyPath) {
    var field = _this.__getField(keyPath, 'getStatusFor');
    return field.resolution && field.resolution.status ? field.resolution.status : null;
  };

  this.getMessageFor = function (keyPath) {
    var field = _this.__getField(keyPath, 'getMessageFor');
    return field.resolution && field.resolution.message ? field.resolution.message : null;
  };

  this.getAsyncStatusFor = function (keyPath) {
    var field = _this.__getField(keyPath, 'getAsyncStatusFor');
    return field.resolution && typeof field.resolution.isProcessing !== 'undefined' ? field.resolution.isProcessing : false;
  };

  this.getSubmissionStatus = function () {
    return _this.__isSubmitting;
  };

  this.getDomIdFor = function (keyPath, value) {
    var layerId = _this.__layerId;
    var field = _this.__getField(keyPath, 'getStatusFor');

    return value ? (0, _ids.buildFieldDomIdWithValue)(layerId, keyPath, value) : field.props.id;
  };

  this.getFieldIdFor = function (keyPath) {
    var field = _this.__getField(keyPath, 'getFieldIdFor');
    return field.props[_Constant2.default.FIELD_ID_DOM_DATA_ATTRIBUTE];
  };

  var state = stateContainer.getState();

  this.__layerId = stateContainer.getLayerId();

  this.__fields = Object.keys(state).reduce(function (fieldsState, fieldStateId) {
    var _extends2;

    var _parseFieldStateId = (0, _ids.parseFieldStateId)(fieldStateId),
        dataType = _parseFieldStateId.dataType,
        fieldId = _parseFieldStateId.fieldId;

    var nextFieldState = dataType === _Constant2.default.FIELD_PROPS_STATE_ID_PREFIX ? { props: state[fieldStateId] } : { resolution: state[fieldStateId] };

    return _extends({}, fieldsState, (_extends2 = {}, _extends2[fieldId] = _extends({}, fieldsState[fieldId], nextFieldState), _extends2));
  }, {});

  this.__getField = function (keyPath, methodName) {
    var layerId = _this.__layerId;
    var fieldId = (0, _ids.buildFieldIdFromUserKeyPath)(keyPath);
    var field = _this.__fields[fieldId];

    if (!field) {
      throw new Error((0, _buildErrorMessage2.default)({
        layerId: layerId,
        fieldId: fieldId,
        message: ['Can\'t find data for field at provided keyPath: ' + JSON.stringify(keyPath), 'Use string to fetch data for non-nested attributes: e.g. `layer.' + methodName + '(\'email\')`', 'Use array to fetch data for nested attributes: e.g. `layer.' + methodName + '([\'user\', \'email\'])`']
      }));
    }

    return field;
  };

  this.__isSubmitting = stateContainer.getIsSubmitting();

  this.__handleDomBlur = handlers.handleDomBlur;
  this.__handleDomChange = handlers.handleDomChange;

  this.notifyOnBlur = handlers.handleCustomBlur;
  this.notifyOnChange = handlers.handleCustomChange;
  this.handleSubmit = handlers.handleSubmit;
};

exports.default = LayerInterface;