'use strict';

exports.__esModule = true;

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _Constant = require('./enums/Constant');

var _Constant2 = _interopRequireDefault(_Constant);

var _DefaultStatus = require('./enums/DefaultStatus');

var _DefaultStatus2 = _interopRequireDefault(_DefaultStatus);

var _validations = require('./modules/validations');

var _utils = require('./modules/validations/utils');

var _ids = require('./modules/ids');

var _utils2 = require('./modules/utils');

var _normalizeFieldsFromProps = require('./modules/normalizeFieldsFromProps');

var _normalizeFieldsFromProps2 = _interopRequireDefault(_normalizeFieldsFromProps);

var _normalizeExternalErrors = require('./modules/normalizeExternalErrors');

var _normalizeExternalErrors2 = _interopRequireDefault(_normalizeExternalErrors);

var _getFieldsPropsState = require('./modules/getFieldsPropsState');

var _getFieldsPropsState2 = _interopRequireDefault(_getFieldsPropsState);

var _buildErrorMessage = require('./modules/buildErrorMessage');

var _buildErrorMessage2 = _interopRequireDefault(_buildErrorMessage);

var _LayerDomHandlers = require('./LayerDomHandlers');

var _LayerDomHandlers2 = _interopRequireDefault(_LayerDomHandlers);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

/**
 * @desc Core component. Container of the validation layer state.
 *
 */
var ValidationLayer = function (_Component) {
  _inherits(ValidationLayer, _Component);

  function ValidationLayer(props) {
    _classCallCheck(this, ValidationLayer);

    for (var _len = arguments.length, rest = Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
      rest[_key - 1] = arguments[_key];
    }

    var _this = _possibleConstructorReturn(this, _Component.call.apply(_Component, [this, props].concat(rest)));

    _initialiseProps.call(_this);

    _this.__layerId = props.id || _Constant2.default.DEFAULT_LAYER_ID;

    _this.__normalizedFields = (0, _normalizeFieldsFromProps2.default)(props.asyncStrategy, props.debounceInterval, props.fields, props.data);
    return _this;
  }

  /**
   * @desc Lifecycle hooks.
   *
   */


  ValidationLayer.prototype.componentWillMount = function componentWillMount() {
    // Set fields props
    this.setNextFieldsPropsState();
  };

  ValidationLayer.prototype.componentDidMount = function componentDidMount() {
    // Performs validation of fields w/ `onMount` strategy
    this.validateOnMount();
  };

  ValidationLayer.prototype.componentWillReceiveProps = function componentWillReceiveProps(nextProps) {
    // Update normalized fields
    this.setNextNormalizedFields(nextProps);

    // Update fields props
    this.setNextFieldsPropsState(nextProps);
  };

  /**
   * @desc Internal getters.
   *
   */


  /**
   * @desc Fields DOM props management.
   *
   */


  /**
   * @desc Normalized fields management.
   *
   */


  /**
   * @desc Blured fields management.
   *
   */


  /**
   * @desc Emitted fields management.
   *
   */


  /**
   * @desc Returns `true` if any ongoing async validation is happening.
   *
   */


  /**
   * @desc Get and set flag if form is submitting.
   *
   */


  /**
   * @desc Get and set flag if form was submitted at least once.
   *
   */


  /**
   * @desc On mount fields validation.
   *
   */


  /**
   * @desc Field validation on `change` & `blur` events.
   *       This method is triggered from <LayerDomHandlers />.
   *
   */


  /**
   * @desc Async validation notifier.
   *       This method is triggered from debouncer,
   *       when async validator is ready to run.
   *
   */


  /**
   * @desc Sets next validation state from various validators.
   *
   */


  /**
   * @desc Sets next deferred validation state from async validator
   *       (if it's not obsolete).
   *
   */


  /**
   * @desc Submission handler.
   *
   */


  /**
   * @desc Form validation on submission.
   *
   */


  /**
   * @desc Handler of form submission failure.
   *       This method is triggered from userland, when form submission failed.
   *       It accepts single argument: errors, reported from API (or anywhere else).
   *
   */


  /**
   * @desc State reset. Triggered from userland.
   *       Back to origins on successful form submission.
   *
   */


  ValidationLayer.prototype.render = function render() {
    var props = this.props;

    var layerId = this.getLayerId();

    if (typeof props.children !== 'function') {
      throw new Error((0, _buildErrorMessage2.default)({
        layerId: layerId,
        message: '`children` must be a function'
      }));
    }

    return _react2.default.createElement(
      _LayerDomHandlers2.default,
      {
        layerId: layerId,
        stateContainer: this,
        data: props.data,
        filter: props.filter,
        handlers: props.handlers,
        transformBeforeStore: props.transformBeforeStore
      },
      function (layer) {
        return props.children && props.children(layer);
      }
    );
  };

  return ValidationLayer;
}(_react.Component);

var _initialiseProps = function _initialiseProps() {
  var _this2 = this;

  this.state = {};
  this.__normalizedFields = [];
  this.__bluredFields = {};
  this.__emittedFields = {};
  this.__isSubmitting = false;
  this.__formWasSubmitted = false;

  this.getLayerId = function () {
    return _this2.__layerId;
  };

  this.getState = function () {
    return _this2.state;
  };

  this.getData = function () {
    return _this2.props.data;
  };

  this.getPropsLevelStrategy = function () {
    return _this2.props.strategy;
  };

  this.getPropsLevelAsyncStrategy = function () {
    return _this2.props.asyncStrategy;
  };

  this.getPropsLevelStatuses = function () {
    return _this2.props.statuses;
  };

  this.getNextFieldsPropsState = function (nextProps) {
    var props = nextProps || _this2.props;

    return (0, _getFieldsPropsState2.default)(_this2.getLayerId(), _this2.getNormalizedFields(), props.data, props.transformBeforeRender, _this2.getIsSubmitting());
  };

  this.setNextFieldsPropsState = function (nextProps, callback) {
    var nextState = _this2.getNextFieldsPropsState(nextProps);
    _this2.setState(nextState, callback);
  };

  this.getNormalizedFields = function () {
    return _this2.__normalizedFields;
  };

  this.getNormalizedField = function (fieldId) {
    var normalizedField = _this2.__normalizedFields.find(function (field) {
      return field.id === fieldId;
    });

    if (!normalizedField) {
      var layerId = _this2.getLayerId();

      throw new Error((0, _buildErrorMessage2.default)({
        layerId: layerId,
        fieldId: fieldId,
        message: ['Can\'t find `field` at key path: ' + fieldId, 'Make sure it exists and has truthy value (e.g. object with options or just `true`).']
      }));
    }

    return normalizedField;
  };

  this.setNextNormalizedFields = function (nextProps) {
    var props = nextProps || _this2.props;

    var nextNormalizedFields = (0, _normalizeFieldsFromProps2.default)(props.asyncStrategy, props.debounceInterval, props.fields, props.data);

    // Adding only new fileds to prevent replacement
    // of debounced async validators
    var normalizedFields = nextNormalizedFields.map(function (nextNormalizedField) {
      var normalizedField = _this2.__normalizedFields.find(function (field) {
        return field.id === nextNormalizedField.id;
      });
      return normalizedField || nextNormalizedField;
    });

    _this2.__normalizedFields = normalizedFields;
  };

  this.getBluredField = function (fieldId) {
    return _this2.__bluredFields[fieldId];
  };

  this.setBluredField = function (fieldId) {
    if (!_this2.__bluredFields[fieldId]) {
      _this2.__bluredFields[fieldId] = true;
    }
  };

  this.getEmittedField = function (fieldId) {
    return _this2.__emittedFields[fieldId];
  };

  this.setEmittedField = function (fieldId) {
    if (!_this2.__emittedFields[fieldId]) {
      _this2.__emittedFields[fieldId] = true;
    }
  };

  this.setAllFieldsEmitted = function () {
    _this2.__emittedFields = _this2.__normalizedFields.map(function (field) {
      return field.id;
    }).reduce(function (fields, fieldId) {
      var _extends2;

      return _extends({}, fields, (_extends2 = {}, _extends2[fieldId] = true, _extends2));
    }, {});
  };

  this.isAnyOngoingAsyncActivity = function () {
    var state = _this2.state;
    // $FlowIgnoreMe: Flow, don't be such an ass

    return Object.keys(state).some(function (stateId) {
      return state[stateId].isProcessing;
    });
  };

  this.getIsSubmitting = function () {
    return _this2.__isSubmitting;
  };

  this.setIsSubmitting = function (isSubmitting) {
    _this2.__isSubmitting = isSubmitting;
  };

  this.getFormWasSubmitted = function () {
    return _this2.__formWasSubmitted;
  };

  this.setFormWasSubmitted = function () {
    if (!_this2.__formWasSubmitted) {
      _this2.__formWasSubmitted = true;
    }
  };

  this.validateOnMount = function () {
    var fields = _this2.props.fields;

    var _performOnMountValida = (0, _validations.performOnMountValidation)(fields, _this2),
        nextSyncState = _performOnMountValida.nextSyncState,
        ongoingAsyncValidations = _performOnMountValida.ongoingAsyncValidations;

    _this2.setNextValidationState(nextSyncState, ongoingAsyncValidations);
  };

  this.validateField = function (field, domData) {
    var _performFieldValidati = (0, _validations.performFieldValidation)(field, domData.value, domData.event, _this2),
        nextSyncState = _performFieldValidati.nextSyncState,
        ongoingAsyncValidations = _performFieldValidati.ongoingAsyncValidations;

    _this2.setNextValidationState(nextSyncState, ongoingAsyncValidations);
  };

  this.notifyAsync = function (fieldId, value, asyncValidation) {
    var stateId = (0, _ids.buildFieldValidationStateId)(fieldId);

    asyncValidation.then(function (results) {
      var deferredNextState = {
        resolution: (0, _utils.buildCompleteAsyncValidationResults)(fieldId, results, _this2.getPropsLevelStatuses(), _this2.getLayerId()),
        forValue: value
      };

      _this2.setNextDeferredValidationState(stateId, deferredNextState);
    });
  };

  this.setNextValidationState = function (nextSyncState, ongoingAsyncValidations) {
    if (Object.keys(nextSyncState).length === 0) return;

    _this2.setState(nextSyncState, function () {
      Object.keys(ongoingAsyncValidations).forEach(function (stateId) {
        ongoingAsyncValidations[stateId].then(function (deferredNextState) {
          return _this2.setNextDeferredValidationState(stateId, deferredNextState);
        });
      });
    });
  };

  this.setNextDeferredValidationState = function (stateId, nextState) {
    var _this2$setState;

    if (!nextState || !nextState.resolution) return;

    var props = _this2.props;

    var _parseFieldStateId = (0, _ids.parseFieldStateId)(stateId),
        fieldId = _parseFieldStateId.fieldId;

    var _parseFieldId = (0, _ids.parseFieldId)(fieldId),
        keyPath = _parseFieldId.keyPath;

    var fieldValue = (0, _utils2.fetchProp)(props.data, keyPath);

    // While async call is being processed,
    // deferred validation state might become obsolete:
    // i.e. value was changed & state was replaced
    // w/ next validation results for the next value
    if (nextState.forValue !== fieldValue) return;

    _this2.setState((_this2$setState = {}, _this2$setState[stateId] = nextState.resolution, _this2$setState));
  };

  this.triggerSubmission = function () {
    // Not submitting anything if there's any ongoing async activity
    // or form is submitting at the moment
    if (_this2.isAnyOngoingAsyncActivity() || _this2.getIsSubmitting()) {
      return;
    }

    _this2.setIsSubmitting(true);
    _this2.setFormWasSubmitted();
    _this2.setAllFieldsEmitted();
    _this2.setNextFieldsPropsState(null, _this2.validateForm);
  };

  this.validateForm = function () {
    var _performOnSubmitValid = (0, _validations.performOnSubmitValidation)(_this2.getData(), _this2.getState(), _this2.getNormalizedFields(), _this2.getPropsLevelStatuses(), _this2.getLayerId()),
        validationState = _performOnSubmitValid.validationState,
        isValid = _performOnSubmitValid.isValid;

    if (isValid) {
      var handlers = _this2.props.handlers;


      if (!handlers || !handlers.onSubmit || !(0, _utils2.isFunction)(handlers.onSubmit)) {
        throw new Error((0, _buildErrorMessage2.default)({
          layerId: _this2.getLayerId(),
          message: "Make sure `handlers.onSubmit` is exists and it's a function"
        }));
      }

      handlers.onSubmit({
        onSuccess: _this2.resetState,
        onFailure: _this2.handleFailurePostSubmitAction
      });
    } else {
      _this2.setState(validationState, _this2.handleFailurePostSubmitAction);
    }
  };

  this.handleFailurePostSubmitAction = function (errors) {
    _this2.setIsSubmitting(false);

    if (!errors) return _this2.setNextFieldsPropsState();

    var nextFieldsPropsState = _this2.getNextFieldsPropsState();

    var statuses = _this2.getPropsLevelStatuses();
    var normalizedErrors = (0, _normalizeExternalErrors2.default)(errors);
    var nextFieldsValidationState = normalizedErrors.reduce(function (nextValidationState, error) {
      var _extends3;

      var stateId = (0, _ids.buildFieldValidationStateId)(error.fieldId);

      return _extends({}, nextValidationState, (_extends3 = {}, _extends3[stateId] = {
        valid: false,
        status: statuses && statuses.failure ? statuses.failure : _DefaultStatus2.default.FAILURE,
        message: error.message,
        isAsync: true
      }, _extends3));
    }, {});

    var nextState = _extends({}, nextFieldsPropsState, nextFieldsValidationState);

    return _this2.setState(nextState);
  };

  this.resetState = function () {
    _this2.setNextNormalizedFields();

    _this2.__bluredFields = {};
    _this2.__emittedFields = {};

    _this2.__isSubmitting = false;
    _this2.__formWasSubmitted = false;

    // All values of the state keys must be set to an empty objects
    // to replace stale data, b/c setState() merges updates w/ current state.
    var resetedState = Object.keys(_this2.state).reduce(function (nextState, key) {
      var _extends4;

      return _extends({}, nextState, (_extends4 = {}, _extends4[key] = {}, _extends4));
    }, {});
    var nextFieldsPropsState = _this2.getNextFieldsPropsState();

    _this2.setState(_extends({}, resetedState, nextFieldsPropsState));
  };
};

exports.default = ValidationLayer;