'use strict';

exports.__esModule = true;
exports.getFieldDomHandler = getFieldDomHandler;
exports.getFieldValueHandler = getFieldValueHandler;

var _buildErrorMessage = require('./buildErrorMessage');

var _buildErrorMessage2 = _interopRequireDefault(_buildErrorMessage);

var _utils = require('./utils');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * @desc It is possible to define DOM handler on the field level
 *       as well as on the props level for all the fields.
 *       This function figures out which one to use for specific field.
 *
 */
function getFieldDomHandler(layerId, fieldId, handlerKey, fieldLevelDomHandlers, propsLevelDomHandlers) {
  var fieldLevelHandler = fieldLevelDomHandlers && fieldLevelDomHandlers[handlerKey];
  var propsLevelHandler = propsLevelDomHandlers && propsLevelDomHandlers[handlerKey];

  var handler = fieldLevelHandler || propsLevelHandler;

  if (handlerKey === 'onChange' && !handler) {
    throw new Error((0, _buildErrorMessage2.default)({
      layerId: layerId,
      fieldId: fieldId,
      message: 'Looks like you forgot to provide `' + handlerKey + '` handler.'
    }));
  }

  if (handler && !(0, _utils.isFunction)(handler)) {
    throw new Error((0, _buildErrorMessage2.default)({
      layerId: layerId,
      fieldId: fieldLevelHandler ? fieldId : null,
      message: '`' + handlerKey + '` must be a function.'
    }));
  }

  return handler;
}

/**
 * @desc Same as function above, but for value handlers,
 *       i.e. `filter`, `transformBeforeStore` & `transformBeforeRender`.
 *
 * NOTE: Might worth to remove this and allow only `field`-level definitions.
 *       Not documenting it for now.
 *
 */


function getFieldValueHandler(layerId, fieldId, handlerKey, fieldLevelValueHandler, propsLevelValueHandler) {
  var handler = fieldLevelValueHandler || propsLevelValueHandler;

  if (handler && !(0, _utils.isFunction)(handler)) {
    throw new Error((0, _buildErrorMessage2.default)({
      layerId: layerId,
      fieldId: fieldLevelValueHandler ? fieldId : null,
      message: '`' + handlerKey + '` must be a function.'
    }));
  }

  return handler;
}