'use strict';

exports.__esModule = true;
exports.getNextDataFromDom = getNextDataFromDom;
exports.getNextDataFromCustomHandler = getNextDataFromCustomHandler;

var _Constant = require('../enums/Constant');

var _Constant2 = _interopRequireDefault(_Constant);

var _ids = require('./ids');

var _buildErrorMessage = require('./buildErrorMessage');

var _buildErrorMessage2 = _interopRequireDefault(_buildErrorMessage);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * @desc When DOM handler is triggered from the view
 *       this function takes event (and some additional data)
 *       and normalizes it to pass to validators & user handlers.
 *
 */
function getNextDataFromDom(layerId, data, event) {
  var domEl = event.target;

  var value = domEl.value,
      checked = domEl.checked;


  var fieldId = domEl.getAttribute(_Constant2.default.FIELD_ID_DOM_DATA_ATTRIBUTE);

  if (!fieldId) {
    throw new Error((0, _buildErrorMessage2.default)({
      layerId: layerId,
      message: "Can't find field id."
    }));
  }

  var _parseFieldId = (0, _ids.parseFieldId)(fieldId),
      attr = _parseFieldId.attr,
      keyPath = _parseFieldId.keyPath;

  return {
    fieldId: fieldId,
    attr: attr,
    keyPath: keyPath,
    value: value,
    checked: checked,
    data: data,
    event: event
  };
}

/**
 * @desc Same as function above except it normalizes the data
 *       received from custom handler.
 *
 */
function getNextDataFromCustomHandler(fieldId, value, data, event) {
  var _parseFieldId2 = (0, _ids.parseFieldId)(fieldId),
      attr = _parseFieldId2.attr,
      keyPath = _parseFieldId2.keyPath;

  return {
    fieldId: fieldId,
    attr: attr,
    keyPath: keyPath,
    value: value,
    data: data,
    event: event
  };
}