'use strict';

exports.__esModule = true;
exports.default = normalizeExternalErrors;

var _ids = require('./ids');

var _utils = require('./utils');

/**
 * @desc Normalizes errors, received from `submitCallbacks.onFailure`.
 *       It happens when API rejected submission and responded w/ errors.
 *
 */
function normalizeExternalErrors(errors) {
  var normalizedErrors = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : [];
  var parentKeyPath = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : [];

  var keys = Object.keys(errors);

  var index = -1;
  // eslint-disable-next-line no-plusplus
  while (++index < keys.length) {
    var key = keys[index];
    var errorsBranch = errors[key];
    var keyPath = parentKeyPath.concat(key);

    if ((0, _utils.isPlainObject)(errorsBranch)) {
      normalizeExternalErrors(errorsBranch, normalizedErrors, keyPath);
    } else {
      normalizedErrors.push({
        fieldId: (0, _ids.buildFieldId)(keyPath),
        message: errorsBranch
      });
    }
  }

  return normalizedErrors;
}