'use strict';

exports.__esModule = true;

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

exports.default = normalizeFieldsFromProps;

var _Constant = require('../enums/Constant');

var _Constant2 = _interopRequireDefault(_Constant);

var _AsyncStrategy = require('../enums/AsyncStrategy');

var _AsyncStrategy2 = _interopRequireDefault(_AsyncStrategy);

var _ids = require('./ids');

var _utils = require('./utils');

var _utils2 = require('./validations/utils');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * @desc Validation Layer accepts fields as Object w/ props
 *       (struture of this Object replicates the structure of the data object).
 *       For internal usage this Object is normalized to flat Array of fields.
 *       Also it wraps async validators with ON_CHANGE strategy in debouncer.
 *
 */
function normalizeFieldsFromProps(propsAsyncStrategy, propsDebounceInterval, fields, data) {
  var normalizedFields = arguments.length > 4 && arguments[4] !== undefined ? arguments[4] : [];
  var parentKeyPath = arguments.length > 5 && arguments[5] !== undefined ? arguments[5] : [];

  var keys = Object.keys(fields);

  var index = -1;
  // eslint-disable-next-line no-plusplus
  while (++index < keys.length) {
    var key = keys[index];
    var fieldsBranch = fields[key];
    var dataBranch = (0, _utils.fetchProp)(data, key);
    var keyPath = parentKeyPath.concat(key);

    if ((0, _utils.isPlainObject)(dataBranch)) {
      normalizeFieldsFromProps(propsAsyncStrategy, propsDebounceInterval, fieldsBranch, dataBranch, normalizedFields, keyPath);
    } else {
      var id = (0, _ids.buildFieldId)(keyPath);

      if ((0, _utils.isPlainObject)(fieldsBranch)) {
        var fieldAsyncStrategy = propsAsyncStrategy || fieldsBranch.asyncStrategy || _AsyncStrategy2.default.DEFAULT;
        var debounceInterval = propsDebounceInterval || fieldsBranch.debounceInterval || _Constant2.default.DEFAULT_DEBOUNCE_INTERVAL;
        var isDebouncedAsyncValidation = fieldsBranch.validateAsync && fieldAsyncStrategy === _AsyncStrategy2.default.ON_CHANGE;

        var normalizedField = isDebouncedAsyncValidation ? _extends({}, fieldsBranch, {
          id: id,
          keyPath: keyPath,
          validateAsync: (0, _utils2.debounce)(fieldsBranch.validateAsync, debounceInterval)
        }) : _extends({}, fieldsBranch, {
          id: id,
          keyPath: keyPath
        });
        normalizedFields.push(normalizedField);
      } else {
        normalizedFields.push({ id: id, keyPath: keyPath });
      }
    }
  }

  return normalizedFields;
}