'use strict';

exports.__esModule = true;
exports.normalizeValueForDom = normalizeValueForDom;
exports.normalizeKeyPath = normalizeKeyPath;
exports.fetchProp = fetchProp;
exports.isDefined = isDefined;
exports.isNull = isNull;
exports.isString = isString;
exports.isNumber = isNumber;
exports.isObject = isObject;
exports.isPlainObject = isPlainObject;
exports.isFunction = isFunction;
exports.isPromise = isPromise;
exports.isChangeEvent = isChangeEvent;
exports.isBlurEvent = isBlurEvent;

/* eslint-disable no-use-before-define */

/**
 * @desc If value is `null` or `undefined`,
 *       then we should pass empty string as a DOM value.
 *
 */
function normalizeValueForDom(value) {
  return isDefined(value) && !isNull(value) ? value : '';
}

/**
 * @desc If key path consists of single attribute,
 *       user can pass it as a string.
 *       This function normalizes it to Array.
 *
 */
function normalizeKeyPath(keyPath) {
  return Array.isArray(keyPath) ? keyPath : [keyPath];
}

/**
 * @desc Retrives value of property at provided key path.
 *       Data container can be vanilla JS Object,
 *       as well as Immutable structure (e.g. Map).
 *
 */
function fetchProp(container, // eslint-disable-line flowtype/no-weak-types
keyPath) {
  var normalizedKeyPath = normalizeKeyPath(keyPath);

  return container.getIn && isFunction(container.getIn) ? container.getIn(normalizedKeyPath) : normalizedKeyPath.reduce(function (data, key) {
    return data[key];
  }, container);
}

function isDefined(subject) {
  return typeof subject !== 'undefined';
}

function isNull(subject) {
  return subject === null;
}

function isString(subject) {
  return typeof subject === 'string';
}

function isNumber(subject) {
  return typeof subject === 'number';
}

function isObject(subject) {
  return typeof subject === 'object';
}

function isPlainObject(subject) {
  return isObject(subject) && Object.prototype.toString.call(subject) === '[object Object]';
}

function isFunction(subject) {
  return typeof subject === 'function';
}

// eslint-disable-next-line flowtype/no-weak-types
function isPromise(subject) {
  return !!subject && (isObject(subject) || isFunction(subject)) && isFunction(subject.then);
}

function isChangeEvent(event) {
  return event.type === 'change';
}

function isBlurEvent(event) {
  return event.type === 'blur';
}